#ifndef SERVERLIB_BI
#define SERVERLIB_BI
/'
 * This is the main header for the TeamSpeak 3 Server SDK. All the functions that are used
 * to communicate with the TeamSpeak 3 Server are here. Please view the documentation for
 * details. Note that some of these functions REQUIRE you to call ts3server_freeMemory()
 * after finishing with the result. This holds true for all the functions returning
 * strings (so the parameter is "char** result"). Again, the details are all in the
 * documentation.
 '/
#include once "public_definitions.bi"


#ifdef __FB_WIN32__
#define LIBNAME "ts3server_win32"
#else
#define LIBNAME "s3server_linux_x86"
#endif

extern "C"

type ServerLibFunctions
  as sub (serverID as uint64, clientID as anyID, voiceData as ubyte ptr, voiceDataSize as uinteger, frequency as uinteger) onVoiceDataEvent
  as sub (serverID as uint64, clientID as anyID) onClientStartTalkingEvent
  as sub (serverID as uint64, clientID as anyID) onClientStopTalkingEvent
  as sub (serverID as uint64, clientID as anyID, channelID as uint64, removeClientError as uinteger ptr) onClientConnected
  as sub (serverID as uint64, clientID as anyID, channelID as uint64) onClientDisconnected
  as sub (serverID as uint64, clientID as anyID, oldChannelID as uint64, newChannelID as uint64) onClientMoved
  as sub (serverID as uint64, invokerClientID as anyID, channelID as uint64) onChannelCreated
  as sub (serverID as uint64, invokerClientID as anyID, channelID as uint64) onChannelEdited
  as sub (serverID as uint64, invokerClientID as anyID, channelID as uint64) onChannelDeleted
  as sub (serverID as uint64, invokerClientID as anyID, textMessage as const zstring ptr) onServerTextMessageEvent
  as sub (serverID as uint64, invokerClientID as anyID, targetChannelID as uint64, textMessage as const zstring ptr) onChannelTextMessageEvent
  as sub (logmessage as const zstring ptr, logLevel as integer, logChannel as const zstring ptr, logID as uint64, logTime as const zstring ptr, completeLogString as const zstring ptr) onUserLoggingMessageEvent
  as sub (serverID as uint64, errorCode as uinteger) onAccountingErrorEvent
  as sub (dataToSend as zstring ptr ptr, sizeOfData as uinteger ptr)onCustomPacketEncryptEvent
  as sub (dataReceived as zstring ptr ptr, dataReceivedSize as uinteger ptr) onCustomPacketDecryptEvent
end type

' Memory management
declare function ts3server_freeMemory(pMem as any ptr) as uinteger

' Construction and Destruction
declare function ts3server_initServerLib             lib LIBNAME (functionPointers as const ServerLibFunctions ptr, usedLogTypes as integer, logFileFolder as const zstring ptr) as uinteger
declare function ts3server_destroyServerLib          lib LIBNAME () as uinteger
declare function ts3server_getServerLibVersion       lib LIBNAME (result as zstring ptr ptr) as uinteger
declare function ts3server_getServerLibVersionNumber lib LIBNAME (result as uint64 ptr) as uinteger

' logging
declare function ts3server_setLogVerbosity lib LIBNAME (logVerbosity as LogLevel) as uinteger

' error handling
declare function ts3server_getGlobalErrorMessage lib LIBNAME (globalErrorCode as uinteger, result as zstring ptr ptr)  as uinteger

' client info
declare function ts3server_getClientVariableAsInt    lib LIBNAME (serverID as uint64, clientID as anyID, flag as ClientProperties, result as integer ptr) as uinteger
declare function ts3server_getClientVariableAsString lib LIBNAME (serverID as uint64, clientID as anyID, flag as ClientProperties, result as zstring ptr ptr) as uinteger
declare function ts3server_setClientVariableAsInt    lib LIBNAME (serverID as uint64, clientID as anyID, flag as ClientProperties, value as integer) as uinteger
declare function ts3server_setClientVariableAsString lib LIBNAME (serverID as uint64, clientID as anyID, flag as ClientProperties, value as const zstring ptr) as uinteger
declare function ts3server_flushClientVariable       lib LIBNAME (serverID as uint64, clientID as anyID) as uinteger

declare function ts3server_setClientWhisperList  lib LIBNAME (serverID as uint64, clID as anyID, channelID as const uint64 ptr, clientID as const anyID ptr) as uinteger

declare function ts3server_getClientList      lib LIBNAME (serverID as uint64, result as anyID ptr ptr) as uinteger
declare function ts3server_getChannelOfClient lib LIBNAME (serverID as uint64, clientID as anyID, result as uint64 ptr) as uinteger
declare function ts3server_clientMove         lib LIBNAME (serverID as uint64, newChannelID as uint64, clientIDArray as const anyID ptr) as uinteger 

' channel info
declare function ts3server_getChannelVariableAsInt    lib LIBNAME (serverID as uint64, channelID as uint64, flag as ChannelProperties, result as integer ptr) as uinteger
declare function ts3server_getChannelVariableAsString lib LIBNAME (serverID as uint64, channelID as uint64, flag as ChannelProperties, result as zstring ptr ptr) as uinteger
declare function ts3server_setChannelVariableAsInt    lib LIBNAME (serverID as uint64, channelID as uint64, flag as ChannelProperties, value as integer) as uinteger
declare function ts3server_setChannelVariableAsString lib LIBNAME (serverID as uint64, channelID as uint64, flag as ChannelProperties, value as const zstring ptr) as uinteger
declare function ts3server_flushChannelVariable       lib LIBNAME (serverID as uint64, channelID as uint64) as uinteger
declare function ts3server_flushChannelCreation       lib LIBNAME (serverID as uint64, channelParentID as uint64, result as uint64 ptr) as uinteger

declare function ts3server_getChannelList            lib LIBNAME (serverID as uint64, result as uint64 ptr ptr) as uinteger
declare function ts3server_getChannelClientList      lib LIBNAME (serverID as uint64, channelID as uint64, result as anyID ptr ptr) as uinteger
declare function ts3server_getParentChannelOfChannel lib LIBNAME (serverID as uint64, channelID as uint64, result as uint64 ptr) as uinteger

declare function ts3server_channelDelete lib LIBNAME (serverID as uint64, channelID as uint64, force as integer) as uinteger
declare function ts3server_channelMove   lib LIBNAME (serverID as uint64, channelID as uint64, newChannelParentID as uint64, newOrder as uint64) as uinteger

' server info
declare function ts3server_getVirtualServerVariableAsInt    lib LIBNAME (serverID as uint64, flag as VirtualServerProperties, result as integer ptr) as uinteger
declare function ts3server_getVirtualServerVariableAsString lib LIBNAME (serverID as uint64, flag as VirtualServerProperties, result as zstring ptr ptr) as uinteger
declare function ts3server_setVirtualServerVariableAsInt    lib LIBNAME (serverID as uint64, flag as VirtualServerProperties, value as integer) as uinteger
declare function ts3server_setVirtualServerVariableAsString lib LIBNAME (serverID as uint64, flag as VirtualServerProperties, value as const zstring ptr) as uinteger
declare function ts3server_flushVirtualServerVariable       lib LIBNAME (serverID as uint64) as uinteger

declare function ts3server_getVirtualServerConnectionVariableAsUInt64 lib LIBNAME (serverID as uint64, flag as ConnectionProperties, result as uint64 ptr) as uinteger
declare function ts3server_getVirtualServerConnectionVariableAsDouble lib LIBNAME (serverID as uint64, flag as ConnectionProperties, result as double ptr) as uinteger

declare function ts3server_getVirtualServerList    lib LIBNAME (result as uint64 ptr ptr) as uinteger
declare function ts3server_stopVirtualServer       lib LIBNAME (serverID as uint64) as uinteger
declare function ts3server_createVirtualServer     lib LIBNAME (serverPort as uinteger, serverIp as const zstring ptr, serverName as const zstring ptr, serverKeyPair as const zstring ptr, serverMaxClients as uinteger, result as uint64 ptr) as uinteger
declare function ts3server_getVirtualServerKeyPair lib LIBNAME (serverID as uint64, result as zstring ptr ptr) as uinteger

end extern

#undef LIBNAME

#endif ' SERVERLIB_BI
